/****************************************************************************
 * Driver for Solarflare network controllers
 *           (including support for SFE4001 10GBT NIC)
 *
 * Copyright 2006:      Solarflare Communications Inc,
 *                      9501 Jeronimo Road, Suite 250,
 *                      Irvine, CA 92618, USA
 *
 * Developed by Solarflare Communications <linux-net-drivers@solarflare.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation, incorporated herein by reference.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ****************************************************************************
 */

#ifndef EFX_RX_H
#define EFX_RX_H

#if defined(EFX_NOT_UPSTREAM) && defined(EFX_USE_SFC_LRO)
#include <linux/skbuff.h>
#endif
#include "net_driver.h"


int efx_probe_rx_queue(struct efx_rx_queue *rx_queue);
void efx_remove_rx_queue(struct efx_rx_queue *rx_queue);
int efx_init_rx_queue(struct efx_rx_queue *rx_queue);
void efx_fini_rx_queue(struct efx_rx_queue *rx_queue);

#if !defined(EFX_NOT_UPSTREAM) || defined(EFX_USE_GENERIC_LRO)
int efx_lro_init(struct net_lro_mgr *lro_mgr, struct efx_nic *efx);
void efx_lro_fini(struct net_lro_mgr *lro_mgr);
#endif
void efx_flush_lro(struct efx_channel *channel);
void efx_rx_strategy(struct efx_channel *channel);
void efx_fast_push_rx_descriptors(struct efx_rx_queue *rx_queue);
void efx_rx_work(struct work_struct *data);
void __efx_rx_packet(struct efx_channel *channel,
		     struct efx_rx_buffer *rx_buf, int checksummed);


#if defined(EFX_NOT_UPSTREAM) && defined(EFX_USE_SFC_LRO)

extern int efx_ssr_init(struct efx_ssr_state *st, struct efx_nic *efx);
extern void efx_ssr_fini(struct efx_ssr_state *st);

extern void __efx_ssr_end_of_burst(struct efx_ssr_state *st);
extern int efx_ssr(struct efx_ssr_state *st, struct efx_rx_buffer *rx_buf);


static inline void efx_ssr_end_of_burst(struct efx_ssr_state *st)
{
	if (!list_empty(&st->active_conns))
		__efx_ssr_end_of_burst(st);
}

#endif /* EFX_USE_SFC_LRO */

#endif /* EFX_RX_H */
